@@ -27,7 +27,7 @@ from flask import (
 )
 from flask_classful import route
 from flask_wtf.csrf import validate_csrf
-from marshmallow import fields, ValidationError, Schema, post_load, pre_load, post_dump
+from marshmallow import fields, ValidationError, post_load, pre_load, post_dump
 from marshmallow.validate import OneOf
 from werkzeug.exceptions import Conflict
 
@@ -42,6 +42,7 @@ from faraday.server.api.base import (
     BulkDeleteMixin,
     BulkUpdateMixin,
 )
+from faraday.server.api.modules.vulns_base import ImpactSchema
 from faraday.server.schemas import (
     PrimaryKeyRelatedField,
     SeverityField,
@@ -59,19 +60,12 @@ vulnerability_template_api = Blueprint('vulnerability_template_api', __name__)
 logger = logging.getLogger(__name__)
 
 
-class ImpactSchema(Schema):
-    accountability = fields.Boolean(attribute='impact_accountability', default=False)
-    availability = fields.Boolean(attribute='impact_availability', default=False)
-    confidentiality = fields.Boolean(attribute='impact_confidentiality', default=False)
-    integrity = fields.Boolean(attribute='impact_integrity', default=False)
-
-
 class CVSS2Schema(AutoSchema):
-    vector_string = fields.String(attribute="cvss2_vector_string")
+    vector_string = fields.String(attribute="cvss2_vector_string", required=False, allow_none=True)
 
 
 class CVSS3Schema(AutoSchema):
-    vector_string = fields.String(attribute="cvss3_vector_string")
+    vector_string = fields.String(attribute="cvss3_vector_string", required=False, allow_none=True)
 
 
 class VulnerabilityTemplateSchema(AutoSchema):
@@ -220,17 +214,10 @@ class VulnerabilityTemplateView(PaginatedMixin,
     get_joinedloads = [VulnerabilityTemplate.creator]
 
     def _envelope_list(self, objects, pagination_metadata=None):
-        vuln_tpls = []
-        for template in objects:
-            vuln_tpls.append({
-                'id': template['_id'],
-                'key': template['_id'],
-                'value': {'rev': ''},
-                'doc': template
-            })
         return {
-            'rows': vuln_tpls,
-            'total_rows': len(objects)
+            'rows': objects,
+            'count': (pagination_metadata.total
+                      if pagination_metadata is not None else len(objects))
         }
 
     def post(self, **kwargs):
