@@ -26,6 +26,10 @@ from faraday.server.models import (
     Workspace,
     User,
     CustomFieldsSchema,
+    Agent,
+    AgentExecution,
+    AgentsSchedule,
+    Credential,
 )
 from faraday.server.utils.search import OPERATORS
 
@@ -62,7 +66,8 @@ class FlaskRestlessFilterSchema(Schema):
     op = fields.String(validate=validate.OneOf(list(OPERATORS.keys())), required=True)
     valid_relationship = {
         'host': Host,
-        'services': Service
+        'services': Service,
+        'workspaces': Workspace
     }
 
     def load(
@@ -94,6 +99,7 @@ class FlaskRestlessFilterSchema(Schema):
             PostgreSQL is very strict with types.
             Return a list of filters (filters are dicts)
         """
+
         if '->' in filter_['name']:
             key = filter_['name'].split('->')[1]
             try:
@@ -105,7 +111,7 @@ class FlaskRestlessFilterSchema(Schema):
 
         if isinstance(filter_['val'], str) and '\x00' in filter_['val']:
             raise ValidationError('Value can\'t contain null chars')
-        if isinstance(filter_['val'], str):
+        if isinstance(filter_['val'], str) and filter_['name'] != 'target':
             if filter_['val'].isnumeric():
                 filter_['val'] = int(filter_['val'])
             else:
@@ -225,6 +231,33 @@ class FlaskRestlessUserFilterSchema(FlaskRestlessFilterSchema):
         return User
 
 
+class FlaskRestlessServiceFilterSchema(FlaskRestlessFilterSchema):
+    def _model_class(self):
+        return Service
+
+
+class FlaskRestlessAgentExecutionSchema(FlaskRestlessFilterSchema):
+
+    def _model_class(self):
+        return AgentExecution
+
+
+class FlaskRestlessAgentSchema(FlaskRestlessFilterSchema):
+
+    def _model_class(self):
+        return Agent
+
+
+class FlaskRestlessAgentSchedulerSchema(FlaskRestlessFilterSchema):
+    def _model_class(self):
+        return AgentsSchedule
+
+
+class FlaskRestlessCredentialFilterSchema(FlaskRestlessFilterSchema):
+    def _model_class(self):
+        return Credential
+
+
 class FlaskRestlessOperator(Schema):
     _or = fields.Nested("self", attribute='or', data_key='or')
     _and = fields.Nested("self", attribute='and', data_key='and')
@@ -235,6 +268,10 @@ class FlaskRestlessOperator(Schema):
         FlaskRestlessWorkspaceFilterSchema,
         FlaskRestlessUserFilterSchema,
         FlaskRestlessVulnerabilityTemplateFilterSchema,
+        FlaskRestlessServiceFilterSchema,
+        FlaskRestlessAgentExecutionSchema,
+        FlaskRestlessAgentSchema,
+        FlaskRestlessCredentialFilterSchema,
     ]
 
     def load(
@@ -288,6 +325,7 @@ class FilterSchema(Schema):
     group_by = fields.List(fields.Nested(FlaskRestlessGroupFieldSchema))
     limit = fields.Integer()
     offset = fields.Integer()
+    columns = fields.List(fields.String())
 
     @post_load
     def validate_order_and_group_by(self, data, **kwargs):
@@ -315,6 +353,10 @@ class FlaskRestlessSchema(Schema):
         FlaskRestlessHostFilterSchema,
         FlaskRestlessWorkspaceFilterSchema,
         FlaskRestlessUserFilterSchema,
+        FlaskRestlessAgentExecutionSchema,
+        FlaskRestlessAgentSchema,
+        FlaskRestlessAgentSchedulerSchema,
+        FlaskRestlessCredentialFilterSchema,
     ]
 
     def load(
