@@ -19,6 +19,7 @@ import inspect
 import logging
 from datetime import date
 
+import sqlalchemy
 # Related third party imports
 from sqlalchemy import (
     and_,
@@ -26,12 +27,15 @@ from sqlalchemy import (
     func,
     nullsfirst,
     nullslast,
-    inspect as sqlalchemy_inspect, text,
+    inspect as sqlalchemy_inspect,
+    text,
 )
 from sqlalchemy.ext.associationproxy import AssociationProxy
 from sqlalchemy.ext.hybrid import hybrid_property
 from sqlalchemy.orm import ColumnProperty
 from sqlalchemy.orm.attributes import InstrumentedAttribute, QueryableAttribute
+from sqlalchemy.sql.elements import BinaryExpression
+from sqlalchemy.sql.schema import Table as SQLAlchemySchemaTableType
 
 # Local application imports
 from faraday.server.models import (
@@ -97,10 +101,18 @@ def get_related_association_proxy_model(attr):
 
 def primary_key_names(model):
     """Returns all the primary keys for a model."""
-    return [key for key, field in inspect.getmembers(model)
-            if isinstance(field, QueryableAttribute)
-            and isinstance(field.property, ColumnProperty)
-            and field.property.columns[0].primary_key]
+    return [
+        key for key, field in inspect.getmembers(model)
+        if isinstance(field, QueryableAttribute)
+        and (
+            isinstance(field.property, ColumnProperty) if hasattr(field, 'property') else False
+        )
+        and (
+            isinstance(field.property, ColumnProperty) if hasattr(field.property, 'columns') else False
+        )
+        and field.property.columns[0].primary_key
+        or isinstance(getattr(model, key, None), hybrid_property)
+    ]
 
 
 def _sub_operator(model, argument, fieldname):
@@ -223,7 +235,9 @@ def get_json_operator(operator):
 
 def get_json_query(table, field, op, op_type, counter):
     if op_type == 'compare':
-        return f"{table}.{field} ->> :key_{counter} {op} :value_{counter}"  # nosec
+        return f"({table}.{field} ->> :key_{counter}) {op} :value_{counter}"  # nosec
+    elif op_type == 'compare_int':
+        return f"({table}.{field} ->> :key_{counter})::int {op} :value_{counter}"  # nosec
     elif op_type == 'exists':
         return f"{table}.{field} ->> :key_{counter} {op}"  # nosec
     elif op_type == 'any':
@@ -421,7 +435,7 @@ class SearchParameters:
     def __repr__(self):
         """Returns a string representation of the search parameters."""
         template = ('<SearchParameters filters={0}, order_by={1}, limit={2},'
-                    ' group_by={3}, offset={4}, junction={5}>')
+                    ' group_by={3}, offset={4}>')
         return template.format(self.filters, self.order_by, self.limit,
                                self.group_by, self.offset)
 
@@ -477,6 +491,7 @@ class QueryBuilder:
     a given model.
 
     """
+
     @staticmethod
     def _create_operation(model, fieldname, operator, argument, relation=None):
         """Translates an operation described as a string to a valid SQLAlchemy
@@ -544,6 +559,10 @@ class QueryBuilder:
                     argument = date.fromisoformat(argument)
                     op_type = 'date'
 
+                if op_type == 'compare':
+                    if custom_field.field_type == 'int':
+                        op_type = 'compare_int'
+
                 increment_bind_counter()
 
                 bindparams = {
@@ -559,7 +578,7 @@ class QueryBuilder:
             opfunc = OPERATORS[operator]
             # In Python 3.0 or later, this should be `inspect.getfullargspec`
             # because `inspect.getargspec` is deprecated.
-            numargs = len(inspect.getargspec(opfunc).args)
+            numargs = len(inspect.getfullargspec(opfunc).args)
             # raises AttributeError if `fieldname` or `relation` does not exist
             field = getattr(model, relation or fieldname)
             # each of these will raise a TypeError if the wrong number of arguments
@@ -707,6 +726,17 @@ class QueryBuilder:
 
         filters = [filt for filt in filters_generator if filt is not None]
 
+        # Check if it is necessary to join the user's table
+        if model.__tablename__ != User.__tablename__:
+            for filter in filters:
+                if isinstance(filter, BinaryExpression):
+                    table = getattr(filter.left, "table", None)
+                    if not isinstance(table, SQLAlchemySchemaTableType):
+                        continue
+                    if table.name == User.__tablename__ and User not in joined_models:
+                        query = query.join(User, model.creator_id == User.id)
+                        joined_models.add(User)
+
         # Multiple filter criteria at the top level of the provided search
         # parameters are interpreted as a conjunction (AND).
         query = query.filter(*filters)
@@ -772,6 +802,66 @@ class QueryBuilder:
 
         return query
 
+    @staticmethod
+    def create_query_delete_returning_only_ids(session, model, search_params, _ignore_order_by=False):
+
+        query = sqlalchemy.delete(model)
+
+        # This function call may raise an exception.
+        valid_model_fields = []
+        for orm_descriptor in sqlalchemy_inspect(model).all_orm_descriptors:
+            if isinstance(orm_descriptor, InstrumentedAttribute):
+                valid_model_fields.append(str(orm_descriptor).split('.')[1])
+            if isinstance(orm_descriptor, hybrid_property):
+                valid_model_fields.append(orm_descriptor.__name__)
+        valid_model_fields += [str(algo).split('.')[1] for algo in sqlalchemy_inspect(model).relationships]
+
+        filters_generator = map(  # pylint: disable=W1636
+            QueryBuilder.create_filters_func(model, valid_model_fields),
+            search_params.filters
+        )
+
+        filters = [filt for filt in filters_generator if filt is not None]
+
+        # Multiple filter criteria at the top level of the provided search
+        # parameters are interpreted as a conjunction (AND).
+        query = query.where(*filters)
+
+        return query
+
+    @staticmethod
+    def create_query_only_ids(session, model, search_params, _ignore_order_by=False):
+
+        query = session.query(model.id)
+
+        # This function call may raise an exception.
+        valid_model_fields = []
+        for orm_descriptor in sqlalchemy_inspect(model).all_orm_descriptors:
+            if isinstance(orm_descriptor, InstrumentedAttribute):
+                valid_model_fields.append(str(orm_descriptor).split('.')[1])
+            if isinstance(orm_descriptor, hybrid_property):
+                valid_model_fields.append(orm_descriptor.__name__)
+        valid_model_fields += [str(algo).split('.')[1] for algo in sqlalchemy_inspect(model).relationships]
+
+        filters_generator = map(  # pylint: disable=W1636
+            QueryBuilder.create_filters_func(model, valid_model_fields),
+            search_params.filters
+        )
+
+        filters = [filt for filt in filters_generator if filt is not None]
+
+        # Multiple filter criteria at the top level of the provided search
+        # parameters are interpreted as a conjunction (AND).
+        query = query.filter(*filters)
+
+        # Apply limit and offset to the query.
+        if search_params.limit:
+            query = query.limit(search_params.limit)
+        if search_params.offset:
+            query = query.offset(search_params.offset)
+
+        return query
+
 
 def create_query(session, model, search_params, _ignore_order_by=False):
     """Returns a SQLAlchemy query object on the given `model` where the search
@@ -842,3 +932,33 @@ def search(session, model, search_params, _ignore_order_by=False):
         return query.one()
     reset_bind_counter()
     return query
+
+
+def create_query_retrieve_only_ids(session, model, search_params):
+    if isinstance(search_params, dict):
+        search_params = SearchParameters.from_dictionary(search_params)
+    return QueryBuilder.create_query_only_ids(session, model, search_params)
+
+
+def search_retrieve_only_ids(session, model, search_params):
+    is_single = search_params.get('single')
+    query = create_query_retrieve_only_ids(session, model, search_params)
+    if is_single:
+        # may raise NoResultFound or MultipleResultsFound
+        return query.one()
+    return query
+
+
+def create_query_delete_returning_only_ids(session, model, search_params):
+    if isinstance(search_params, dict):
+        search_params = SearchParameters.from_dictionary(search_params)
+    return QueryBuilder.create_query_delete_returning_only_ids(session, model, search_params)
+
+
+def delete_returning_only_ids(session, model, search_params):
+    is_single = search_params.get('single')
+    query = create_query_delete_returning_only_ids(session, model, search_params)
+    if is_single:
+        # may raise NoResultFound or MultipleResultsFound
+        return query.one()
+    return query
